
/*
 * This file is part of EKG/gEKG, a software package to record EKG.
 * Copyright  2003 Lars Stollenwerk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "EKGfreq.h"

// ----------------------------------------------------------------------
//
// Implementation EKGdevice
// 
// ----------------------------------------------------------------------

/*
 * Der Puls wird anhand von Kanal I bestimmt.
 *                .
 *               / \          Schwelle
 *  - - - - - - /| |\- - - - - - - - -
 *             / | | \                 
 * -----------'| | | |'---------------
 *     A       |B|C|D|    A
 *
 * Bereiche eines Pulses:
 * ----------------------
 * A - unterhalb der Schwelle, QRS ist aus
 *      *  keine besondere Operation
 * B - oberhalb der Schwelle, QRS ist aus
 *      *  QRS wird eingeschaltet
 *      *  Pulsfrequenz neu berechnen
 *      *  es werden die geforderten Signale gegeben.
 * C - oberhalb der Schwelle, QRS ist an
 *      *  keine besondere Operation
 * D - unterhalb der Schwelle, QRS ist an
 *      *  QRS wird ausgeschaltet
 *
 * Stndige Operation:
 * -------------------
 * - Pulse zerfallen mit decay.
 * - Schwellen werden mitgefhrt.
 */
void EKGfreq::input( float * d )
{
  if ( !QRS && aboveThreshold( d[0] ) ) // Bereich B
  {
    QRS = true;
    handleQRS();
  }
  else if ( QRS && !aboveThreshold( d[0] ) ) // Bereich D
    QRS = false;
  // Stndige Operation
  Pthreshold *= decay;
  Nthreshold *= decay;
  if ( d[0] > Pthreshold ) Pthreshold = d[0];
  if ( d[0] < Nthreshold ) Nthreshold = d[0];
  // Daten an nchstes Gert weritergeben. 
  EKGdevice::input( d );
}

// ----------------------------------------------------------------------
//
// public Methoden
// 
// ----------------------------------------------------------------------

/*
 * Piepsen ein/aus schalten 
 */
bool EKGfreq::setBeep( bool x )
{
  bool old = beepFlag;
  beepFlag = x;
  return old;
}


/*
 * Pulsausgabe ein/aus schalten 
 */
bool EKGfreq::setFreq( bool x )
{
  bool old = freqFlag;
  freqFlag = x;
  return old;
}


/**
 * Zeitkonstante setzen. Der Faktor in jedem Zeitschritt <tt>dt</tt> betrgt 
 * <code>
 *         decay = ( 1 - dt / x )
 * </code>
 * Das entspricht der DGL 
 * <code>
 *         d/dt u(t) = - u(t) / x
 * </code>
 */
float EKGfreq::setDecay( float x )
{
  if ( x < EKG_TimeStep ) 
    throw string("EKGfreq::setDecay : decay time must be > EKG_TimeStep");
  float old = decay;
  decay = 1 - EKG_TimeStep / x;
  return EKG_TimeStep / ( 1 - old );
}

// ----------------------------------------------------------------------
//
// protected Methoden
//
// ----------------------------------------------------------------------


/*
 * reagiert auf QRS Komplex.
 */
void EKGfreq::handleQRS()
{
  calcFreq();
  beep();
  printFreq();
  cerr.flush();
}

/*
 * prft ber/unterschreiten der positiven/negativen Schwelle
 */
bool EKGfreq::aboveThreshold( const float x )
{
  if ( Pthreshold > -Nthreshold )
    return ( x > Pthreshold );
  else
    return ( x < Nthreshold );
}


/*
 * ermittelt Pulsfrequenz
 */
void EKGfreq::calcFreq()
{
  struct timeval tv;
  struct timezone tz;
  double now; 

  gettimeofday( &tv, &tz );
  now = tv.tv_sec + 1e-6 * tv.tv_usec;
  freq = (int)floor( 60 / ( now - lastQRS ) + 0.5 ) ;
  lastQRS = now;
}

