
/*
 * This file is part of EKG/gEKG, a software package to record EKG.
 * Copyright  2003 Lars Stollenwerk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef EKGLFILTER_H
#define EKGLFILTER_H

#include <string>
#include <cmath>
#include "calibrate.h"
#include "EKGfilter.h"

/**
 * Diese Klasse implementiert einen Tiefpafilter. Der Tiefpafilter 2.
 * Ordnung dient der Rauschunterdrckung. 
 * <p>
 * <b>Tiefpafilter</b>
 * <p>
 * Der Tiefpafilter ist einem aktiven Filter 2. Ordnung nachempfunden.
 <pre>
		   +------------------------+
		   |		---         |
		   |             |          |
		   |        C2 -----   C1 -----
                   |           -----      -----
         R1        |   R2        |   |\     |
 e o-----/\/\/\/---+---/\/\/\/---+---|+\    |
   o                                 |  >---+------o a
   |                             +---|-/    |      o
   |				 |   |/     |      |
  ---				 +----------+     ---
 </pre>
 * Der Filter gengt der DGL:
 <pre>
    a'' = - K a' + L ( e - a ) 
 </pre>
 * mit 
 <pre>
    K = 1 / ( C1 ( R1 || R2 ) )
    L = 1 / ( R1 R2 C1 C2 )
 </pre>
 * Fr eine sinvolle Filtercharakteristik werden die Bauteile wie folgt
 * gewhlt:
 <pre>
    R1 = R2 = R
    C1 = C * 2/d
    C2 = C * d/2
 </pre>
 * Daraus ergibt sich fr die Parameter der DGL
 <pre>
    K = d / RC = d * omega
    L = 1 / (RC)^2 = omega^2
 </pre>
 * Die Grenzfrequenz des Filters ist
 <pre>
   f = omega / 2pi = 1 / ( 2pi * RC )
 </pre>
 * Die Dmpfung <tt>d</tt> bestimmt die Filtercharakteristik.
 <pre>
 d = 0          Hier setzt Oszillation ein!
 d = 1          Tschebyscheff:  steile Flanke, Welligkeit
 d = sqrt( 2 )  Butterworth:  flacher Amplitudenverlauf
 d = sqrt( 3 )  Bessel:  optimales Laufzeitverhalten, nicht so steile Falnke
 </pre>
 * Es wird ein Tschebyscheff-Filter implementiert.  Die Frequenz kann mit
 * setLFreq() gesetzt werden und ist auf 10 Hz voreingestellt. 
 *
 * @author Lars Stollenwerk, Oktober 2002
 */
class EKGlfilter : public EKGfilter
{
  public:
    /**
     * Konstruktor
     */
    EKGlfilter()
    {
      setLFreq( 10 );
      La[0] = La[1] = La[2] = 0;
      LA[0] = LA[1] = LA[2] = 0; 
      oldd[0] = oldd[1] = oldd[2] = 0; 
    }
    /**
     * Grenzfrequenz des Tiefpafilters setzen. 
     * @param x neue Frequenz in Hz.
     * @return alter Wert
     */
    float setLFreq( float x );
    /// Grenzfrequenz erfragen
    float getLFreq() { return sqrt( LL / EKG_TimeStep ) / 2 / M_PI; } 
    /**
     * berschreiben von EKGdevice::input(). Hier werden die DGL fr die
     * Filter numerisch gelst.
     * <p>
     * <b>Tiefpafilter</b>
     * <p>
     * Die DGL <tt> a'' = - K a' + L ( e - a ) </tt> wird in zwei
     * Komponenten umgeschrieben:
     <pre>
     a' = A
     A' = - K A + L ( e - a )
     </pre>
     * und in Differenzengleichungen berfhrt. TS ist die Zeitschrittweite
     * EKG_TimeStep.
     <pre>
     Delta a = TS * A
     Delta A = - TS K A + TS L ( e - a )
     </pre>
     */
    virtual void input( float * d );
  private:
    // ----------------------------------------------------------------------
    // Tiefpa
    // ----------------------------------------------------------------------
    /// Konstante K fr Tiefpa. <tt> LK = EKG_TimeStep * K </tt>
    float LK;
    /// Konstante L fr Tiefpa. <tt> LL = EKG_TimeStep * L </tt>
    float LL;
    /// Ausgang a des Teifpa.
    float La[3];
    /// Zeitableitung des Ausgangs a vom Tiefpa.
    float LA[3];
    /// Alter Eingangswert fr Oversampling
    float oldd[3];
    /// Zwischenschritte fr das Oversampling
    static const int oversampling = 8;
};

#endif
